{******************************************************************************}
{                                                                              }
{       Mecenas Library - Delphi 6                                             }
{       mcLibD6                                                                }
{       Copyright (c) 2000-2002, Ivan Mecenas                                  }
{                                                                              }
{******************************************************************************}

unit mcDCLLib;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs, Math;

  function mcRetornaAno(Data :TDateTime) :Word;
  function mcRetornaMes(Data :TDateTime) :Word;
  function mcRetornaDia(Data :TDateTime) :Word;
  function mcDiaSemana(Data :TDateTime) :String;
  function mcPrimeiroDiaMes(Data :TDateTime) :TDateTime;
  function mcUltimoDiaMes(Data :TDateTime) :TDateTime;
  function mcDataExtensoLonga(Data :TDateTime) :String;
  function mcDataExtensoCurta(Data :TDateTime) :String;
  function mcMesExtenso(Data :TDateTime; Abreviado :Boolean=FALSE) :String;
  function mcFeriadoMovel(Ano :Integer; Fer :String='DOMINGO') :TDateTime;

implementation

function mcDataExtensoLonga(Data :TDateTime) :String;
var
  Ano, Mes, Dia :Word;
begin
  DecodeDate(Data,Ano,Mes,Dia);
  Result := IntToStr(Dia)+' de '+mcMesExtenso(Data,False)+' de '+IntToStr(Ano);
end;

function mcDataExtensoCurta(Data :TDateTime) :String;
var
  Ano, Mes, Dia :Word;
begin
  DecodeDate(Data,Ano,Mes,Dia);
  Result := IntToStr(Dia)+' '+mcMesExtenso(Data,TRUE)+' '+IntToStr(Ano);
end;

function mcDiaSemana(Data :TDateTime) :String;
begin
  case DayOfWeek(Data) of
    1: mcDiaSemana := 'Domingo';
    2: mcDiaSemana := 'Segunda';
    3: mcDiaSemana := 'Tera';
    4: mcDiaSemana := 'Quarta';
    5: mcDiaSemana := 'Quinta';
    6: mcDiaSemana := 'Sexta';
    7: mcDiaSemana := 'Sbado';
  end;
end;

function mcMesExtenso(Data :TDateTime; Abreviado :Boolean=FALSE) :String;
const
  Meses :Array[1..12] of PChar = ('Janeiro' ,'Fevereiro','Maro'   ,'Abril'    ,
                                  'Maio'    ,'Junho'    ,'Julho'   ,'Agosto'   ,
                                  'Setembro','Outubro'  ,'Novembro','Dezembro');
var
  Ano, Mes, Dia :Word;
begin
  DecodeDate(Data,Ano,Mes,Dia);
  Result := Meses[Mes];
  if Abreviado then Result := AnsiUpperCase(Copy(Result,1,3));
end;

function mcPrimeiroDiaMes(Data :TDateTime) :TDateTime;
var
  Ano, Mes, Dia :Word;
begin
  DecodeDate(Data,Ano,Mes,Dia);
  Result := EncodeDate(Ano,Mes,01);
end;

function mcRetornaAno(Data :TDateTime) :Word;
var
  Mes, Dia :Word;
begin
  DecodeDate(Data,Result,Mes,Dia);
end;

function mcRetornaDia(Data :TDateTime) :Word;
var
  Ano, Mes :Word;
begin
  DecodeDate(Data,Ano,Mes,Result);
end;

function mcRetornaMes(Data :TDateTime) :Word;
var
  Ano, Dia :Word;
begin
  DecodeDate(Data,Ano,Result,Dia);
end;

function mcUltimoDiaMes(Data :TDateTime) :TDateTime;
var
  Ano, Mes, Dia :Word;
  DtUltDia      :TDateTime;
begin
  DecodeDate(Data,Ano,Mes,Dia);
  DtUltDia := (Data - Dia ) + 33;
  DecodeDate(DtUltDia,Ano,Mes,Dia);
  Result   := DtUltDia - Dia;
end;

function mcFeriadoMovel(Ano :Integer; Fer :String='DOMINGO') :TDateTime;
const
  FERIADOS :Array[0..6] of PChar = ('DOMINGO','QUINTA','SEXTA','CORPUS','SEGUNDA','TERCA','QUARTA');
var
  a, b, c, d, e :Integer;
  i,n           :Integer;
  Mes, Dia      :Integer;
begin
  n := 0;

  for i := 0 to 6 do
    if AnsiUpperCase(Fer) = FERIADOS[i] then n := i;

  a :=             Ano mod  4;
  b :=             Ano mod  7;
  c :=             Ano mod 19;
  d := (19*c+24)       mod 30;
  e := (2*a+4*b+6*d+5) mod  7;

  if (22+d+e) > 31 then
    begin
      Dia := d+e-9;
      Mes := 4;
    end
  else
    begin
      Dia := d+e+22;
      Mes := 3;
    end;

  Result := EncodeDate(Ano,Mes,Dia);

  case n of
    0:                    ; //Domingo de Pscoa (default)
    1: Result := Result-3 ; //Quinta-Feira Santa
    2: Result := Result-2 ; //Sexta-Feira Santa
    3: Result := Result+60; //Corpus Christ
    4: Result := Result-48; //Segunda-Feira de Carnaval
    5: Result := Result-47; //Tera-Feira de Carnaval
    6: Result := Result-46; //Quarta-Feira de Cinzas
  end;
end;

end.
